﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Markup;
using Microsoft.Xaml.Interactivity;

namespace CommandInvokeDemo.Command
{
    /// <summary>
    /// 带参数的Action
    /// </summary>
    [ContentProperty(Name = "Parameters")]
    public sealed class WdCallMethodAction : DependencyObject, IAction
    {
        public static readonly DependencyProperty MethodNameProperty = DependencyProperty.Register("MethodName",
            typeof (string), typeof (WdCallMethodAction), new PropertyMetadata(null));

        public static readonly DependencyProperty TargetObjectProperty = DependencyProperty.Register("TargetObject",
            typeof (object), typeof (WdCallMethodAction), new PropertyMetadata(null));

        public static readonly DependencyProperty ParametersProperty = DependencyProperty.Register(
            "Parameters", typeof (ICollection<Parameter>), typeof (WdCallMethodAction),
            new PropertyMetadata(new List<Parameter>()));
        
        /// <summary>
        /// 方法名：参数有?,eventArgs,sender,dataContext
        ///     eg：Test
        ///     eg：Test(?,?)
        ///     eg：Test(sender,?,?)
        /// </summary>
        public string MethodName
        {
            get { return (string) GetValue(MethodNameProperty); }
            set { SetValue(MethodNameProperty, value); }
        }

        public ICollection<Parameter> Parameters
        {
            get { return (ICollection<Parameter>) GetValue(ParametersProperty); }
            set { SetValue(ParametersProperty, value); }
        }

        public object TargetObject
        {
            get { return GetValue(TargetObjectProperty); }
            set { SetValue(TargetObjectProperty, value); }
        }

        public object Execute(object sender, object parameter)
        {
            InvokeMethod(MethodName, sender, parameter, TargetObject, Parameters);
            return true;
        }

        public void InvokeMethod(string methodName, object sender, object eventArgs, object dataContext,
            ICollection<Parameter> parameters)
        {
            var start = methodName.IndexOf('(');
            var end = methodName.IndexOf(')');

            MethodInfo methodInfo;

            if (start >= 0 && end >= 0)
            {
                var paras = methodName.Substring(start + 1, end - start - 1)
                    .Split(new[] {',', ' '}, StringSplitOptions.RemoveEmptyEntries);

                methodName = MethodName.Substring(0, start);

                var allParameter = new List<object>();
                var enumerator = parameters.GetEnumerator();
                foreach (var para in paras)
                {
                    switch (para)
                    {
                        case "?":
                            enumerator.MoveNext();
                            allParameter.Add(enumerator.Current);
                            break;
                        case "eventArgs":
                            allParameter.Add(eventArgs);
                            break;
                        case "sender":
                            allParameter.Add(sender);
                            break;
                        case "dataContext":
                            allParameter.Add(dataContext);
                            break;
                        default:
                            throw new NotImplementedException(string.Format("没有实现该参数:{0}", para));

                    }
                }

                var paramCount = paras.Length;
                methodInfo = TargetObject.GetType().GetRuntimeMethods()
                    .FirstOrDefault(m => Equals(m.Name, methodName) && m.GetParameters().Count() == paramCount);

                methodInfo.Invoke(TargetObject, allParameter.ToArray());
            }
            else
            {
                methodInfo = TargetObject.GetType().GetRuntimeMethod(methodName, new Type[0]);
                methodInfo.Invoke(TargetObject, null);
            }

        }
    }
}
